package TimTools;

use v5.30;
use strict;
use warnings;

=head1 NAME

TimTools - Some common functions.

=cut

our $VERSION = '0.09';

=head1 SYNOPSIS

Nothing to show. Just an umbrella module.

=head1 AUTHOR

Tim Potapov, C<< <tim.potapov[AT]gmail.com> >>

=head1 SUPPORT

You can find documentation for this module with the perldoc command.

    perldoc TimTools

You can also look for information at:

=head1 LICENSE AND COPYRIGHT

This software is Copyright (c) 2022 by Tim Potapov.

This is free software, licensed under:

  The Artistic License 2.0 (GPL Compatible)

=cut

1;    # End of TimTools
